@@ -1,9 +1,9 @@
 package Net::DNS::RR;
 
 #
-# $Id: RR.pm 1552 2017-03-13 09:44:07Z willem $
+# $Id: RR.pm 1569 2017-05-31 09:01:09Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1552 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1569 $)[1];
 
 
 =head1 NAME
@@ -176,7 +176,7 @@ sections required for certain dynamic update operations.
 sub _new_hash {
 	my ( $base, %argument ) = @_;
 
-	my %attribute = ( owner => '.', type => 'ANY' );
+	my %attribute = ( owner => '.', type => 'NULL' );
 	while ( my ( $key, $value ) = each %argument ) {
 		$attribute{lc $key} = $value;
 	}
@@ -192,6 +192,8 @@ sub _new_hash {
 	$self->class($class) if defined $class;			# specify CLASS
 	$self->ttl($ttl)     if defined $ttl;			# specify TTL
 
+	die "type $type not implemented" if $populated && ref($self) eq __PACKAGE__;
+
 	while ( my ( $attribute, $value ) = each %attribute ) {
 		$self->$attribute( ref($value) eq 'ARRAY' ? @$value : $value );
 	}
@@ -472,7 +474,6 @@ Resource record time to live in seconds.
 # published API.  These are required for parsing BIND zone files but
 # should not be used in other contexts.
 my %unit = ( W => 604800, D => 86400, H => 3600, M => 60, S => 1 );
-%unit = ( %unit, map /\D/ ? lc($_) : $_, %unit );
 
 sub ttl {
 	my ( $self, $time ) = @_;
@@ -482,7 +483,7 @@ sub ttl {
 	my $ttl = 0;
 	my %time = reverse split /(\D)\D*/, $time . 'S';
 	while ( my ( $u, $t ) = each %time ) {
-		my $scale = $unit{$u} || die qq(bad time: $t$u);
+		my $scale = $unit{uc $u} || die qq(bad time: $t$u);
 		$ttl += $t * $scale;
 	}
 	$self->{ttl} = $ttl;
@@ -517,7 +518,7 @@ sub _format_rdata {			## format rdata portion of RR string
 
 sub _parse_rdata {			## parse RR attributes in argument list
 	my $self = shift;
-	die join ' ', $self->type, 'not implemented' if ref($self) eq __PACKAGE__;
+	die join ' ', 'type', $self->type, 'not implemented' if ref($self) eq __PACKAGE__;
 	die join ' ', 'no zone file representation defined for', $self->type;
 }
 
@@ -673,15 +674,14 @@ our %_MINIMAL = ( 'ANY' => bless ['type' => 255], __PACKAGE__ );
 our %_LOADED = %_MINIMAL;
 
 sub _subclass {
-	my $class   = shift;
-	my $rrname  = shift;
-	my $default = shift;
+	my ( $class, $rrname, $default ) = @_;
 
 	unless ( $_LOADED{$rrname} ) {
-		local @INC = LIB;
 		my $rrtype = typebyname($rrname);
 
 		unless ( $_LOADED{$rrtype} ) {			# load once only
+			local @INC = LIB;
+
 			my $mnemon = typebyval($rrtype);
 			$mnemon =~ s/[^A-Za-z0-9]//g;		# expect the unexpected
 
@@ -763,17 +763,16 @@ sub AUTOLOAD {				## Default method
 	my $module = join '::', __PACKAGE__, typebyval( $self->{type} );
 	eval("require $module") if $oref eq __PACKAGE__;
 
-	@_ = (<<"END");
-***  FATAL PROGRAM ERROR!!	Unknown method '$method'
+	@_ = ( <<"END", $@, "@object" );
+***  FATAL PROGRAM ERROR!!	Unknown instance method '$method'
 ***  which the program has attempted to call for the object:
 ***
 $string
 ***
-***  @object has no instance method '$method'
-***  $@
 ***  THIS IS A BUG IN THE CALLING SOFTWARE, which incorrectly assumes
 ***  that the object would be of a particular type.  The type of an
 ***  object should be checked before calling any of its methods.
+***
 END
 	goto &{'Carp::confess'};
 }
