@@ -1,13 +1,13 @@
 package Net::DNS::RR::AFSDB;
 
 #
-# $Id: AFSDB.pm 1390 2015-09-11 11:42:11Z willem $
+# $Id: AFSDB.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1390 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -68,7 +68,7 @@ sub hostname {
 	my $self = shift;
 
 	$self->{hostname} = new Net::DNS::DomainName2535(shift) if scalar @_;
-	$self->{hostname}->name if defined wantarray && $self->{hostname};
+	$self->{hostname}->name if $self->{hostname};
 }
 
 
