@@ -1,9 +1,9 @@
 package Net::DNS::RR::CAA;
 
 #
-# $Id: CAA.pm 1528 2017-01-18 21:44:58Z willem $
+# $Id: CAA.pm 1561 2017-04-19 13:08:13Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1561 $)[1];
 
 
 use strict;
@@ -77,8 +77,7 @@ sub critical {
 	my $bit = 0x0080;
 	for ( shift->{flags} ) {
 		my $set = $bit | ( $_ ||= 0 );
-		return $bit & $_ unless scalar @_;
-		$_ = (shift) ? $set : ( $set ^ $bit );
+		$_ = (shift) ? $set : ( $set ^ $bit ) if scalar @_;
 		return $_ & $bit;
 	}
 }
