@@ -1,9 +1,9 @@
 package Net::DNS::RR::CDNSKEY;
 
 #
-# $Id: CDNSKEY.pm 1584 2017-07-28 16:15:17Z willem $
+# $Id: CDNSKEY.pm 1580 2017-06-26 11:44:49Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1584 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1580 $)[1];
 
 
 use strict;
@@ -20,21 +20,19 @@ Net::DNS::RR::CDNSKEY - DNS CDNSKEY resource record
 use integer;
 
 
-sub _format_rdata {			## format rdata portion of RR string.
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return $self->SUPER::_format_rdata()
-			if scalar grep $_, @{$self}{qw(flags algorithm)};
-	return defined $self->{algorithm} ? '0 3 0 0' : '';	# RFC8078 mandated notation
+	return $self->SUPER::_encode_rdata() if $self->{algorithm};
+	return defined $self->{algorithm} ? pack( 'xxH*x', '03' ) : '';
 }
 
 
-sub _parse_rdata {			## populate RR from rdata in argument list
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return $self->SUPER::_parse_rdata(@_) if $_[2];
-	die 'invalid RDATA (DNSKEY delete)'   if "@_" !~ m/^0 3 0/;    # ignore key
-	$self->algorithm(0);
+	return $self->SUPER::_format_rdata() if $self->{algorithm};
+	return defined $self->{algorithm} ? '0 3 0 0' : '';
 }
 
 
@@ -42,8 +40,8 @@ sub algorithm {
 	my ( $self, $arg ) = @_;
 
 	return $self->SUPER::algorithm($arg) if $arg;
-	return $self->SUPER::algorithm() unless defined $arg;
-	@{$self}{qw(flags protocol algorithm keybin)} = ( 0, 3, 0, '' );
+	@{$self}{qw(flags protocol algorithm keybin)} = ( 0, 3, 0, '' ) if defined $arg;
+	return $self->SUPER::algorithm();
 }
 
 
@@ -106,6 +104,6 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, L<Net::DNS::RR::DNSKEY>, RFC7344, RFC8078
+L<perl>, L<Net::DNS>, L<Net::DNS::RR>, L<Net::DNS::RR::DNSKEY>, RFC7344, RFC8087
 
 =cut
