@@ -1,13 +1,13 @@
 package Net::DNS::RR::HIP;
 
 #
-# $Id: HIP.pm 1512 2016-10-26 09:14:27Z willem $
+# $Id: HIP.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1512 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -82,8 +82,9 @@ sub pkalgorithm {
 
 sub hit {
 	my $self = shift;
+	my @args = map { /[^0-9A-Fa-f]/ ? croak "corrupt hexadecimal" : $_ } @_;
 
-	$self->hitbin( pack "H*", map { die "!hex!" if m/[^0-9A-Fa-f]/; $_ } join "", @_ ) if scalar @_;
+	$self->hitbin( pack "H*", join "", @args ) if scalar @args;
 	unpack "H*", $self->hitbin() if defined wantarray;
 }
 
