@@ -1,13 +1,13 @@
 package Net::DNS::RR::NID;
 
 #
-# $Id: NID.pm 1390 2015-09-11 11:42:11Z willem $
+# $Id: NID.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1390 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -66,8 +66,7 @@ sub nodeid {
 
 	$self->{nodeid} = pack 'n4', map hex($_), split /:/, $idnt if defined $idnt;
 
-	sprintf '%0.4x:%0.4x:%0.4x:%0.4x', unpack 'n4', $self->{nodeid}
-			if defined wantarray && $self->{nodeid};
+	sprintf '%0.4x:%0.4x:%0.4x:%0.4x', unpack 'n4', $self->{nodeid} if $self->{nodeid};
 }
 
 
