@@ -1,13 +1,13 @@
 package Net::DNS::RR::NS;
 
 #
-# $Id: NS.pm 1390 2015-09-11 11:42:11Z willem $
+# $Id: NS.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1390 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -56,7 +56,7 @@ sub nsdname {
 	my $self = shift;
 
 	$self->{nsdname} = new Net::DNS::DomainName1035(shift) if scalar @_;
-	$self->{nsdname}->name if defined wantarray && $self->{nsdname};
+	$self->{nsdname}->name if $self->{nsdname};
 }
 
 
