@@ -1,9 +1,9 @@
 package Net::DNS::RR::NSEC3;
 
 #
-# $Id: NSEC3.pm 1528 2017-01-18 21:44:58Z willem $
+# $Id: NSEC3.pm 1567 2017-05-19 09:52:52Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1567 $)[1];
 
 
 use strict;
@@ -37,19 +37,17 @@ my %digest = (
 		'SHA-1' => 1,					# RFC3658
 		);
 
-	my @digestbyalias = ( 'SHA' => 1 );
+	my @digestalias = ( 'SHA' => 1 );
 
 	my %digestbyval = reverse @digestbyname;
 
-	my @digestbynum = map { ( $_, 0 + $_ ) } keys %digestbyval;    # accept algorithm number
-
-	my %digestbyname = map { s /[^A-Za-z0-9]//g; $_ } @digestbyalias, @digestbyname, @digestbynum;
-
+	my @digestrehash = map /^\d/ ? ($_) x 3 : do { s/[\W]//g; uc($_) }, @digestbyname, @digestalias;
+	my %digestbyname = @digestrehash;			# work around broken cperl
 
 	sub _digestbyname {
 		my $name = shift;
 		my $key	 = uc $name;				# synthetic key
-		$key =~ s /[^A-Z0-9]//g;			# strip non-alphanumerics
+		$key =~ s /[\W_]//g;				# strip non-alphanumerics
 		$digestbyname{$key} || croak "unknown digest type $name";
 	}
 
@@ -78,7 +76,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless defined $self->{typebm};
+	return '' unless defined $self->{hnxtname};
 	my $salt = $self->saltbin;
 	my $hash = $self->{hnxtname};
 	pack 'CCn C a* C a* a*', $self->algorithm, $self->flags, $self->iterations,
@@ -91,7 +89,7 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless defined $self->{typebm};
+	return '' unless defined $self->{hnxtname};
 	my @rdata = (
 		$self->algorithm, $self->flags, $self->iterations,
 		$self->salt || '-', $self->hnxtname, $self->typelist
@@ -145,8 +143,7 @@ sub optout {
 	my $bit = 0x01;
 	for ( shift->{flags} ) {
 		my $set = $bit | ( $_ ||= 0 );
-		return $bit & $_ unless scalar @_;
-		$_ = (shift) ? $set : ( $set ^ $bit );
+		$_ = (shift) ? $set : ( $set ^ $bit ) if scalar @_;
 		return $_ & $bit;
 	}
 }
@@ -179,8 +176,8 @@ sub saltbin {
 
 sub hnxtname {
 	my $self = shift;
-	$self->{hnxtname} = _decode_base32(shift) if scalar @_;
-	_encode_base32( $self->{hnxtname} ) if defined wantarray;
+	$self->{hnxtname} = _decode_base32hex(shift) if scalar @_;
+	_encode_base32hex( $self->{hnxtname} ) if defined wantarray;
 }
 
 
@@ -189,24 +186,21 @@ sub covered {
 	my $name = shift;
 
 	# first test if the domain name is in the NSEC3 zone.
-	my @domainlabels = new Net::DNS::DomainName($name)->_wire;
 	my ( $owner, @zonelabels ) = $self->{owner}->_wire;
-	my $ownerhash = _decode_base32($owner);
-
+	my @labels = new Net::DNS::DomainName( lc $name )->_wire;
 	foreach ( reverse @zonelabels ) {
-		return 0 unless lc($_) eq lc( pop(@domainlabels) || return 0 );
+		tr /\101-\132/\141-\172/;
+		return 0 unless $_ eq ( pop(@labels) || '' );
 	}
 
+	my $ownerhash = _decode_base32hex($owner);
+	my $nexthash  = "$self->{hnxtname}";
+
 	my $namehash = _hash( $self->algorithm, $name, $self->iterations, $self->saltbin );
-	my $nexthash = "$self->{hnxtname}";
 
-	unless ( $ownerhash lt $nexthash ) {			# last or only NSEC3 RR
-		return 1 if $namehash lt $nexthash;
-		return $namehash gt $ownerhash;
-	}
-
-	return 0 unless $namehash gt $ownerhash;		# general case
-	return $namehash lt $nexthash;
+	my $c1 = $namehash cmp $ownerhash;
+	my $c2 = $nexthash cmp $namehash;
+	return ( $c1 + $c2 ) == 2;
 }
 
 
@@ -214,20 +208,18 @@ sub match {
 	my $self = shift;
 	my $name = shift;
 
-	my $namehash = _hash( $self->algorithm, $name, $self->iterations, $self->saltbin );
-
 	my ($owner) = $self->{owner}->_wire;
-	my $ownerhash = _decode_base32($owner);
+	my $ownerhash = _decode_base32hex($owner);
 
-	$namehash eq $ownerhash;
+	$ownerhash eq _hash( $self->algorithm, $name, $self->iterations, $self->saltbin );
 }
 
 
 ########################################
 
-sub _decode_base32 {
+sub _decode_base32hex {
 	local $_ = shift || '';
-	tr [0-9a-vA-V] [\000-\037\012-\037];
+	tr [0-9A-Va-v\060-\071\101-\126\141-\166] [\000-\037\012-\037\000-\037\012-\037];
 	$_ = unpack 'B*', $_;
 	s/000(.....)/$1/g;
 	my $l = length;
@@ -236,7 +228,7 @@ sub _decode_base32 {
 }
 
 
-sub _encode_base32 {
+sub _encode_base32hex {
 	local $_ = unpack 'B*', shift;
 	s/(.....)/000$1/g;
 	my $l = length;
@@ -272,7 +264,7 @@ sub _hash {
 }
 
 
-sub name2hash { _encode_base32(&_hash); }			# uncoverable pod
+sub name2hash { _encode_base32hex(&_hash); }			# uncoverable pod
 
 
 sub hashalgo { &algorithm; }					# uncoverable pod
@@ -409,7 +401,9 @@ matches as defined in the NSEC3 specification:
 
 =head1 COPYRIGHT
 
-Copyright (c)2007,2008 NLnet Labs.  Author Olaf M. Kolkman
+Copyright (c)2017 Dick Franks
+
+Portions Copyright (c)2007,2008 NLnet Labs.  Author Olaf M. Kolkman
 
 All rights reserved.
 
