@@ -1,13 +1,13 @@
 package Net::DNS::RR::X25;
 
 #
-# $Id: X25.pm 1382 2015-08-26 20:39:33Z willem $
+# $Id: X25.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -57,7 +57,7 @@ sub address {
 	my $self = shift;
 
 	$self->{address} = new Net::DNS::Text(shift) if scalar @_;
-	$self->{address}->value if defined wantarray && $self->{address};
+	$self->{address}->value if $self->{address};
 }
 
 
@@ -89,7 +89,6 @@ other unpredictable behaviour.
 
 =head2 PSDNaddress
 
-
 =head2 address
 
     $address = $rr->address;
